#Final perms:


# Define the variables
$appName = "SaaSAssure For MS OneDrive"
$redirectUri = "https://ui.na.saasassure.com/connector-add"
$secretExpiryDate = (Get-Date).AddYears(100).ToString("yyyy-MM-dd")
$outputSecretExpiry = (Get-Date).AddYears(100).AddDays(-1).ToString("yyyy-MM-dd")
$resourceAppId = "00000003-0000-0000-c000-000000000000"  # Microsoft Graph API ID

# Login to Azure
az login --allow-no-subscriptions

# Validate if App already exists
$appExists = az ad app list --display-name $appName | ConvertFrom-Json
if ($appExists.Count -gt 0) {
    Write-Host "Application already exists. Please remove it and try again."
    Read-Host -Prompt "Press 'Enter' to exit" | Out-Null
    exit
}

# Get the tenant ID and subscription ID
$tenantId = (az account show --query tenantId -o tsv).Trim()
$subscriptionId = (az account show --query id -o tsv).Trim()

# Set the subscription
az account set --subscription $subscriptionId

# Create the app registration and retrieve the app ID
echo "Creating OneDrive App for SaaSAssure in $subscriptionId ..."
$app = az ad app create --display-name $appName --web-redirect-uris $redirectUri | ConvertFrom-Json
$appId = $app.appId

# Define the required resource accesses using Microsoft Graph API application permissions
$requiredResourceAccesses = @"
{
    "requiredResourceAccess": [
        {
            "resourceAppId": "$resourceAppId",
            "resourceAccess": [
                { "id": "19dbc75e-c2e2-444c-a770-ec69d8559fc7", "type": "Role" },   # Directory.ReadWrite.All
                { "id": "75359482-378d-4052-8f01-80520e7db3cd", "type": "Role" }   # Files.ReadWrite.All

            ]
        }
    ]
}
"@

# Write the JSON to a file to avoid command line parsing issues
$filePath = "tempRequiredAccesses.json"
$requiredResourceAccesses | Out-File -FilePath $filePath -Encoding UTF8

try {
    # Update the app with required resource accesses using the file
    echo "Setting up the required permissions to backup OneDrive with SaaSAssure ..."
    az ad app update --id $appId --required-resource-accesses $filePath

    # Sleep command to allow time for Azure to create the App and permissions
    Start-Sleep -s 30

    # Admin consent for all permissions
    echo "Granting admin consent for the required permissions (Must be logged in as GA)..."
    az ad app permission admin-consent --id $appId
} finally {
    # Remove the temporary file
    Remove-Item $filePath -Force
}

# Create a secret for the app with an expiry stated in secretExpiryDate variable (100 years) 
$secret = az ad app credential reset --id $appId --append --end-date $secretExpiryDate --display-name SaaSAssure | ConvertFrom-Json

# Get the secret password
$secretPassword = $secret.password

#Get Current Working Path
$currentworkingpath = Get-Location


# Save the client ID and secret to a text file
$output = "Client ID: $appId `nSecret: $secretPassword `nExpiry: $outputSecretExpiry `nTenant: $tenantId"
$output | Out-File -FilePath "AzureAppCredentialsOneDrive.txt"

# Get the current working directory
$currentworkingpath = Get-Location

# Print the current working directory
Write-Host "You can find your credentials in this location:" 
echo ""
Write-Host "$currentworkingpath\AzureAppCredentialsOneDrive.txt"
echo ""
Write-Host "Please paste this path into your file explorer on your local machine to locate AzureAppCredentialsOneDrive.txt"
echo ""

# Pause the script at the end
Read-Host -Prompt "Press Enter to exit"

echo " "