#Final perms:


# Define the variables
$appName = "SaaSAssure For MS Teams"
$redirectUri = "https://ui.na.saasassure.com/connector-add"
$secretExpiryDate = (Get-Date).AddYears(100).ToString("yyyy-MM-dd")
$outputSecretExpiry = (Get-Date).AddYears(100).AddDays(-1).ToString("yyyy-MM-dd")
$resourceAppId = "00000003-0000-0000-c000-000000000000"  # Microsoft Graph API ID

# Login to Azure
az login --allow-no-subscriptions

# Validate if App already exists
$appExists = az ad app list --display-name $appName | ConvertFrom-Json
if ($appExists.Count -gt 0) {
    Write-Host "Application already exists. Please remove it and try again."
    Read-Host -Prompt "Press 'Enter' to exit" | Out-Null
    exit
}

# Get the tenant ID and subscription ID
$tenantId = (az account show --query tenantId -o tsv).Trim()
$subscriptionId = (az account show --query id -o tsv).Trim()

# Set the subscription
az account set --subscription $subscriptionId

# Create the app registration and retrieve the app ID
echo "Creating Teams App for SaaSAssure in $subscriptionId ..."
$app = az ad app create --display-name $appName --web-redirect-uris $redirectUri | ConvertFrom-Json
$appId = $app.appId

# Define the required resource accesses using Microsoft Graph API application permissions
$requiredResourceAccesses = @"
{
    "requiredResourceAccess": [
        {
            "resourceAppId": "$resourceAppId",
            "resourceAccess": [
                { "id": "dc149144-f292-421e-b185-5953f2e98d7f", "type": "Role" },   # AppCatalog.ReadWrite.All
                { "id": "a2611786-80b3-417e-adaa-707d4261a5f0", "type": "Role" },   # CallRecord-PstnCalls.Read.All
                { "id": "45bbb07e-7321-4fd7-a8f6-3ff27e6a81c8", "type": "Role" },   # CallRecords.Read.All
                { "id": "f3a65bd4-b703-46df-8f7e-0174fea562aa", "type": "Role" },   # Channel.Create
                { "id": "59a6b24b-4225-4393-8165-ebaec5f55d7a", "type": "Role" },   # Channel.ReadBasic.All
                { "id": "35930dcf-aceb-4bd1-b99a-8ffed403c974", "type": "Role" },   # ChannelMember.ReadWrite.All
                { "id": "7b2449af-6ccd-4f4d-9f78-e550c193f0d1", "type": "Role" },   # ChannelMessage.Read.All
                { "id": "4d02b0cc-d90b-441f-8d82-4fb55c34d6bb", "type": "Role" },   # ChannelMessage.UpdatePolicyViolation.All
                { "id": "243cded2-bd16-4fd6-a953-ff8177894c3d", "type": "Role" },   # ChannelSettings.ReadWrite.All
                { "id": "294ce7c9-31ba-490a-ad7d-97a7d075e4ed", "type": "Role" },   # Chat.ReadWrite.All
                { "id": "57257249-34ce-4810-a8a2-a03adf0c5693", "type": "Role" },   # ChatMember.ReadWrite.All
                { "id": "19dbc75e-c2e2-444c-a770-ec69d8559fc7", "type": "Role" },   # Directory.ReadWrite.All
                { "id": "dbaae8cf-10b5-4b86-a4a1-f871c94c6695", "type": "Role" },   # GroupMember.ReadWrite.All
                { "id": "6931bccd-447a-43d1-b442-00a195474933", "type": "Role" },   # MailboxSettings.ReadWrite
                { "id": "83cded22-8297-4ff6-a7fa-e97e9545a259", "type": "Role" },   # Presence.ReadWrite.All
                { "id": "b7760610-0545-4e8a-9ec3-cce9e63db01c", "type": "Role" },   # Schedule.ReadWrite.All
                { "id": "0121dc95-1b9f-4aed-8bac-58c5ac466691", "type": "Role" },   # TeamMember.ReadWrite.All
                { "id": "bdd80a03-d9bc-451d-b7c4-ce7c63fe3c8f", "type": "Role" },   # TeamSettings.ReadWrite.All
                { "id": "70dec828-f620-4914-aa83-a29117306807", "type": "Role" },   # TeamActivity.Read.All
                { "id": "a96d855f-016b-47d7-b51c-1218a98d791c", "type": "Role" },   # TeamsTab.ReadWrite.All
                { "id": "23fc2474-f741-46ce-8465-674744c5c361", "type": "Role" }    # Team.Create
            ]
        }
    ]
}
"@

# Write the JSON to a file to avoid command line parsing issues
$filePath = "tempRequiredAccesses.json"
$requiredResourceAccesses | Out-File -FilePath $filePath -Encoding UTF8

try {
    # Update the app with required resource accesses using the file
    echo "Setting up the required permissions to backup Teams with SaaSAssure ..."
    az ad app update --id $appId --required-resource-accesses $filePath

    # Sleep command to allow time for Azure to create the App and permissions
    Start-Sleep -s 30

    # Admin consent for all permissions
    echo "Granting admin consent for the required permissions (Must be logged in as GA)..."
    az ad app permission admin-consent --id $appId
} finally {
    # Remove the temporary file
    Remove-Item $filePath -Force
}

# Create a secret for the app with an expiry stated in secretExpiryDate variable (100 years) 
$secret = az ad app credential reset --id $appId --append --end-date $secretExpiryDate --display-name SaaSAssure | ConvertFrom-Json

# Get the secret password
$secretPassword = $secret.password

# Save the client ID and secret to a text file
$output = "Client ID: $appId `nSecret: $secretPassword `nExpiry: $outputSecretExpiry `nTenant: $tenantId"
$output | Out-File -FilePath "AzureAppCredentialsTeams.txt"

# Get the current working directory
$currentworkingpath = Get-Location

# Print the current working directory
Write-Host "You can find your credentials in this location:" 
echo ""
Write-Host "$currentworkingpath\AzureAppCredentialsTeams.txt"
echo ""
Write-Host "Please paste this path into your file explorer on your local machine to locate AzureAppCredentialsTeams.txt"
echo ""

# Pause the script at the end
Read-Host -Prompt "Press Enter to exit"

echo " "